""" Maintain the Roster """
import cPickle
import os

import BLooper
import BDirectory
import BEntry
import BNode
import BNodeInfo
import BFile
import BMessage
import BAlert

from InterfaceKit import B_WARNING_ALERT
from StorageKit import *
from SupportKit import *

from modules.constants import *
import modules.people
from modules.log import log
from modules.util import *

class Worker(BLooper.BLooper):
	def __init__(self):
		BLooper.BLooper.__init__(self, 'Contact Worker')
		self._cleanup()
		self.Run()

	def QuitRequested(self):
		# Remove status attributes on quit
		self._cleanup()
		log('Contact Worker quit')
		return True

	def MessageReceived(self, msg):
		if msg.what == INBOUND_ROSTER:
			action = msg.FindString('action')
			jid = msg.FindString('jid')
			info = msg.FindString('info')
			info = cPickle.loads(info)
			if action == 'update':
				entry = modules.people.getEntryForJID(jid)
				if entry is not None:
					node = BNode.BNode(entry)
					if info.has_key('online'):
						node.WriteAttr('Jabber:Online', B_STRING_TYPE,0,'%s\0' % info['online'])
					if info.has_key('show'):
						if info['show'] is not None:
							node.WriteAttr('Jabber:Show', B_STRING_TYPE,0,'%s\0' % info['show'])
						else:
							node.WriteAttr('Jabber:Show', B_STRING_TYPE,0,'available\0')
					if info.has_key('status'):
						if info['status'] is not None:
							node.WriteAttr('Jabber:Status', B_STRING_TYPE,0,'%s\0' % info['status'])
						else:
							node.WriteAttr('Jabber:Status', B_STRING_TYPE,0,'\0')
				else:
					balert = BAlert.BAlert('Unknown Contact', 'No People file found for %s' % jid, 'Ok, I will fix it later', None, None, None, B_WARNING_ALERT)
					balert.Go(None)
					log('No people file found for %s.' % jid)
			elif action == 'add':
				log('add: %s %s' % (jid, info))
				bentry = modules.people.getEntryForJID(jid)
				if bentry is None:
					if not info.has_key('name'):
						contactname = jid
					else:
						contactname = info['name']
					if contactname.strip() == '':
						contactname = jid
					self._createPerson(jid, contactname)
					log('created new person file for %s' % jid)
			elif action == 'remove':
				log('remove: %s %s' % (jid, info))
				self._removeAttributes(jid)
			else:
				log('unknown action: %s' % action)
		elif msg.what == STATUS_DISCONNECTED:
			self._cleanup()

	def _cleanup(self):
		modules.people.clearAll()

	def _createPerson(self, jid, name):
		dir = '/boot/home/people'
		fpath = os.path.join(dir,name)
		Done = False
		ix = 1
		while not Done:
			if not os.path.exists(fpath):
				Done = True
			else:
				fpath = os.path.join(dir,'%s-%s' % (name, ix))
				ix += 1
		bfile = BFile.BFile(fpath, B_READ_WRITE | B_CREATE_FILE | B_FAIL_IF_EXISTS)
		bfile.InitCheck()
		bnode = BNode.BNode(fpath)
		bnode.InitCheck()
		bnodeinfo = BNodeInfo.BNodeInfo(bnode)
		bnodeinfo.SetType('application/x-person')
		bnode.WriteAttr('Jabber:JID', B_STRING_TYPE,0,'%s\0' % jid)
		bnode.WriteAttr('META:name', B_STRING_TYPE,0,'%s\0' % name)

	def _removeAttributes(self, jid):
		bentry = modules.people.getEntryForJID(jid)
		attributes = ['Jabber:Online','Jabber:Show','Jabber:Status']
		bnode = BNode.BNode(bentry)
		for attr in attributes:
			try:
				bnode.RemoveAttr(attr)
			except BNode.error:
				pass
