""" Main """

__author__    = "Lars Hansson (lars@unet.net.ph)"
__version__   = "$Revision: 0.9 $"
__date__      = "$Date: 2003/02/25 $"

import sys
import time
import os.path
import traceback
import copy

import BApplication
import BAlert
import BMessage
import BMessenger
from AppKit import *

from modules.log import log
from modules.constants import *
from modules.config import Account
import modules.ui

from Control.window import ControlWindow
import Contact.worker
import Message.worker
import Jabber.worker

# BeOS Application
class MainApp(BApplication.BApplication):
	OUTBOUND_CONNECTION_MSGS = [OUTBOUND_PRESENCE,MSG_CONNECT,MSG_DISCONNECT]
	def __init__(self, sig):
		BApplication.BApplication.__init__(self, sig)

	def ReadyToRun(self):
		self.connectionworker = Jabber.worker.Worker()
		self.contactworker = Contact.worker.Worker()
		self.messageworker = Message.worker.Worker()
		self.window = ControlWindow(100, 80)
		self.window.Show()

	def MessageReceived(self, msg):
		# Roster
		if msg.what == INBOUND_ROSTER:
			self.contactworker.PostMessage(msg)
		# Message
		elif msg.what == INBOUND_MESSAGE:
			if msg.FindString('type') != 'error':
				self.messageworker.PostMessage(msg)
			else:
				errorstr = msg.FindString('errorstring')
				errorcode = msg.FindString('errorcode')
				mfrom = msg.FindString('from')
				modules.ui.ErrorBox('%s %s %s' % (mfrom, errorcode, errorstr))
		elif msg.what == OUTBOUND_MESSAGE:
			self.connectionworker.PostMessage(msg)
		# Connection Messages
		elif msg.what in self.OUTBOUND_CONNECTION_MSGS:
			self.connectionworker.PostMessage(msg)
		# Status Messages
		elif msg.what == STATUS_CONNECTED:
			self.window.PostMessage(msg)
			self.contactworker.PostMessage(msg)
		elif msg.what == STATUS_DISCONNECTED:
			self.window.PostMessage(msg)
			self.contactworker.PostMessage(msg)
		# Presence
		elif msg.what == INBOUND_PRESENCE:
			self.window.PostMessage(msg)
		# Errors
		elif msg.what == STATUS_ERROR:
			modules.ui.ErrorBox(msg.FindString('errstr'))
		elif msg.what == ROSTER_ITEM_REQUEST:
			self.connectionworker.PostMessage(msg)
		elif msg.what == B_SILENT_RELAUNCH:
			self.window.Activate(True)

	def QuitRequested(self):
		log('Waiting for workers to quit...')
		self.connectionworker.PostMessage(B_QUIT_REQUESTED)
		self.contactworker.PostMessage(B_QUIT_REQUESTED)
		self.messageworker.PostMessage(B_QUIT_REQUESTED)
		time.sleep(3)
		return True

# Main
def main():
	# Start the App	
	ThisApp = MainApp(SIG_HERMOD)
	try:
		ThisApp.Run()
	except:
		print 'blah'
		trace = traceback.extract_stack()[-2]
		msg = '%s() [%s:%i]' % (trace[FUNCNAME], os.path.split(trace[FILENAME])[-1], trace[LINENO])
		log(msg)
		modules.ui.ErrorBox(msg, True)
		try:
			# try to quit cleanly
			ThisApp.PostMessage(B_QUIT_REQUSTED)
		except:
			pass

# Do the Mario
if __name__ == '__main__':
	main()
