"""Roster Window"""
import time

import BWindow
import BBox
import BListView
import BScrollView
import BStringItem
import BFilePanel

from InterfaceKit import *
from StorageKit import *

from menus import MainMenu
import roster
import util
from constants import *
from modules.ui import *
from modules.config import Account
import modules.people

ENABLE_EXPORT = 7000
DISABLE_EXPORT = 7001
ENABLE_MERGE = 7003
DISABLE_MERGE = 7004

class RosterWindow(BWindow.BWindow):
	"""The control window"""
	def __init__(self, x,y):
		BWindow.BWindow.__init__(self, (x,y,0,0), 'Unknown Contacts', B_TITLED_WINDOW,
			B_QUIT_ON_WINDOW_CLOSE|B_ASYNCHRONOUS_CONTROLS)
		bounds = self.Bounds()
		# Main menubar
		bar = MainMenu(bounds)
		self.AddChild(bar)
		self.bar = bar
		barbounds = bar.Bounds()
		box = BBox.BBox((0,barbounds[3]+1,bounds[2],bounds[3]), 'boundingbox', B_FOLLOW_ALL, B_FULL_UPDATE_ON_RESIZE|B_WILL_DRAW, B_NO_BORDER)
		boxbounds = box.Bounds()
		boxframe = box.Frame()
		self.AddChild(box)
		clist = ContactList((1,1,boxbounds[2]-1-B_V_SCROLL_BAR_WIDTH,boxbounds[3]))
		bsv = BScrollView.BScrollView('contactscroll', clist, B_FOLLOW_ALL, B_WILL_DRAW, False, True, B_PLAIN_BORDER)
		box.AddChild(bsv)
		#box.AddChild(clist)
		self.ResizeTo(250.0,200.0)

	def MessageReceived(self, msg):
		if msg.what == RM_GET_ROSTER:
			lv = self.FindView('contactlist')
			lv.MakeEmpty()
			acc = Account(msg.FindString('account'))
			if acc.password is None:
				win = ModalPrompt('Password Prompt','Password')
				password = win.Go()
			else:
				password = acc.password
			r = roster.Roster(acc.user, password, acc.host, acc.port, acc.ssl)
			contactlist = r.get()
			if contactlist is not None:
				lv = self.FindView('contactlist')
				lv.populateList(contactlist)
			else:
				modules.ui.ErrorBox('Wrong username/password or connection error', True)
		elif msg.what == DISABLE_EXPORT:
			item = self.bar.FindItem('Export')
			item.SetEnabled(False)
		elif msg.what == ENABLE_EXPORT:
			item = self.bar.FindItem('Export')
			item.SetEnabled(True)
		elif msg.what == DISABLE_MERGE:
			item = self.bar.FindItem('Merge')
			item.SetEnabled(False)
		elif msg.what == ENABLE_MERGE:
			item = self.bar.FindItem('Merge')
			item.SetEnabled(True)
		elif msg.what == RM_EXPORT_CONTACTS:
			lv = self.FindView('contactlist')
			ix = 0
			while lv.CurrentSelection(ix) is not -1:
				item = lv.ItemAt(lv.CurrentSelection(ix))
				util.export(item.name, item.jid, '/boot/home/people')
				ix += 1
			ix = 0
			while lv.CurrentSelection() is not -1:
				lv.RemoveItem(lv.CurrentSelection())
		elif msg.what == RM_MERGE_CONTACT:
			lv = self.FindView('contactlist')
			cursel = lv.CurrentSelection()
			item = lv.ItemAt(cursel)
			bmsg = BMessage.BMessage(RM_FP_MSG)
			bmsg.AddString('jid', '%s' % item.jid)
			bmsg.AddInt32('cursel', cursel)
			self.fp = BFilePanel.BFilePanel(B_OPEN_PANEL,None,None,0,False,bmsg,None,True)
			self.fp.SetPanelDirectory("/boot/home/people")
			self.fp.Show()
		elif msg.what == RM_FP_MSG:
			ref = msg.FindRef('refs')
			jid = msg.FindString('jid')
			util.merge(jid, ref)
			lv = self.FindView('contactlist')
			lv.RemoveItem(msg.FindInt32('cursel'))

class ContactList(BListView.BListView):
	def __init__(self, rect):
		BListView.BListView.__init__(self,rect,'contactlist', B_MULTIPLE_SELECTION_LIST, B_FOLLOW_ALL_SIDES)

	def populateList(self, contacts):
		for contact in contacts:
			bentry = modules.people.getEntryForJID(contact[0])
			if bentry is None:
				item = ContactItem(contact[1], contact[0])
				self.AddItem(item)

	def SelectionChanged(self):
		count = 0
		cursel = self.CurrentSelection()
		if cursel == -1:
			count = cursel
		else:
			while self.CurrentSelection(count) is not -1:
				count += 1
		if count == -1:
			bmsg = BMessage.BMessage(DISABLE_EXPORT)
			self.Window().PostMessage(bmsg)
			bmsg = BMessage.BMessage(DISABLE_MERGE)
			self.Window().PostMessage(bmsg)
		elif count == 1:
			bmsg = BMessage.BMessage(ENABLE_EXPORT)
			self.Window().PostMessage(bmsg)
			bmsg = BMessage.BMessage(ENABLE_MERGE)
			self.Window().PostMessage(bmsg)
		else:
			bmsg = BMessage.BMessage(ENABLE_EXPORT)
			self.Window().PostMessage(bmsg)
			bmsg = BMessage.BMessage(DISABLE_MERGE)
			self.Window().PostMessage(bmsg)


class ContactItem(BStringItem.BStringItem):
	"""Feed item"""
	def __init__(self, name,jid):
		BStringItem.BStringItem.__init__(self, '%s (%s)' % (name, jid))
		self.jid = jid
		self.name = name
	def getURL(self):
		return self.url
